@@ -1,8 +1,8 @@
 /*
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
-        the Free Software Foundation; You may only use version 2 of the License,
-        you have no option to use any other version.
+        the Free Software Foundation; either version 2, or (at your option)
+        any later version.
  
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -44,6 +44,7 @@
 #include "keyboard.h"
 #include "mypixmap.h"
 #include "mywindow.h"
+#include "screen.h"
 #include "settings.h"
 
 #define ANY                             0
@@ -141,9 +142,9 @@
 #define ALL_WORKSPACES                 (int) 0xFFFFFFFF
 
 #define CONSTRAINED_WINDOW(c)          ((c->win_layer > WIN_LAYER_DESKTOP) && \
-					(c->win_layer < WIN_LAYER_ABOVE_DOCK) && \
-				       !(c->type & (WINDOW_DESKTOP | WINDOW_DOCK)) && \
-				       !(c->legacy_fullscreen))
+                                        (c->win_layer < WIN_LAYER_ABOVE_DOCK) && \
+                                       !(c->type & (WINDOW_DESKTOP | WINDOW_DOCK)) && \
+                                       !(c->legacy_fullscreen))
 
 #define WINDOW_TYPE_DIALOG               (WINDOW_DIALOG | WINDOW_MODAL_DIALOG)
 #define WINDOW_TYPE_DONT_PLACE           (WINDOW_DESKTOP | WINDOW_DOCK | WINDOW_SPLASHSCREEN)
@@ -159,6 +160,7 @@
 
 #define CLIENT_CAN_HIDE_WINDOW(c)       (!(c->transient_for) && FLAG_TEST_AND_NOT(c->flags, CLIENT_FLAG_HAS_HIDE, CLIENT_FLAG_SKIP_TASKBAR))
 #define CLIENT_CAN_MAXIMIZE_WINDOW(c)   FLAG_TEST_ALL(c->flags, CLIENT_FLAG_HAS_MAXIMIZE | CLIENT_FLAG_HAS_RESIZE | CLIENT_FLAG_IS_RESIZABLE)
+#define CLIENT_CAN_STICK_WINDOW(c)      (!(c->transient_for) && FLAG_TEST_AND_NOT(c->flags, CLIENT_FLAG_HAS_STICK, CLIENT_FLAG_SKIP_TASKBAR))
 
 typedef enum
 {
@@ -189,6 +191,9 @@ struct _ClientPixmapCache
 
 struct _Client
 {
+    /* Reference to our screen structure */
+    ScreenData *md;
+    
     Window window;
     Window frame;
     Window transient_for;
@@ -252,40 +257,31 @@ extern Client *clients;
 extern unsigned int client_count;
 
 void clientClearLastOpTime (Client * c);
-void clientSetNetState (Client *);
 void clientUpdateWinState (Client *, XClientMessageEvent *);
-void clientUpdateNetState (Client *, XClientMessageEvent *);
-void clientGetNetWmType (Client *);
 void clientCoordGravitate (Client *, int, int *, int *);
 void clientGravitate (Client *, int);
 void clientConfigure (Client *, XWindowChanges *, int, unsigned short);
 void clientGetMWMHints (Client *, gboolean);
 void clientGetWMNormalHints (Client *, gboolean);
 void clientGetWMProtocols (Client *);
-void clientFocusNew(Client *);
 void clientClearPixmapCache (Client *);
-void clientFrame (Window, gboolean);
+void clientFrame (ScreenData *, Window, gboolean);
 void clientUnframe (Client *, gboolean);
-void clientFrameAll ();
-void clientUnframeAll ();
-void clientGetNetStruts (Client *);
+void clientFrameAll (ScreenData *);
+void clientUnframeAll (ScreenData *);
 void clientInstallColormaps (Client *);
 void clientUpdateColormaps (Client *);
-void clientUpdateAllFrames (gboolean);
+void clientUpdateAllFrames (ScreenData *, gboolean);
 void clientGrabKeys (Client *);
 void clientUngrabKeys (Client *);
 void clientGrabButtons (Client *);
 void clientUngrabButtons (Client *);
-void clientPassGrabButton1(Client *);
 Client *clientGetFromWindow (Window, int);
 Client *clientAtPosition (int, int, Client *);
-Client *clientGetNext (Client *, int);
-Client *clientGetPrevious (Client *, int);
-void clientPassFocus (Client *);
 void clientShow (Client *, gboolean);
 void clientHide (Client *, int, gboolean);
 void clientHideAll (Client *, int);
-void clientToggleShowDesktop (gboolean);
+void clientToggleShowDesktop (ScreenData *, gboolean);
 void clientClose (Client *);
 void clientKill (Client *);
 void clientEnterContextMenuState (Client *);
@@ -304,17 +300,12 @@ void clientToggleAbove (Client *);
 void clientToggleBelow (Client *);
 void clientRemoveMaximizeFlag (Client *);
 void clientToggleMaximized (Client *, int);
-gboolean clientAcceptFocus (Client * c);
-void clientUpdateFocus (Client *, unsigned short);
-void clientSetFocus (Client *, unsigned short);
-Client *clientGetFocus ();
 void clientScreenResize(void);
 void clientMove (Client *, XEvent *);
 void clientResize (Client *, int, XEvent *);
 void clientCycle (Client *, XEvent *);
 void clientButtonPress (Client *, Window, XButtonEvent *);
 Client *clientGetLeader (Client *);
-GList *clientGetStackList (void);
 #ifdef HAVE_LIBSTARTUP_NOTIFICATION
 char *clientGetStartupId (Client *);
 #endif
