@@ -1,8 +1,8 @@
 /*
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
-        the Free Software Foundation; You may only use version 2 of the License,
-        you have no option to use any other version.
+        the Free Software Foundation; either version 2, or (at your option)
+        any later version.
  
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,7 +15,7 @@
  
         oroborus - (c) 2001 Ken Lynch
         Metacity - (c) 2001 Havoc Pennington
-        xfwm4    - (c) 2002-2003 Olivier Fourdan
+        xfwm4    - (c) 2002-2004 Olivier Fourdan
  
  */
 
@@ -23,29 +23,20 @@
 #include <config.h>
 #endif
 
-#ifdef GDK_MULTIHEAD_SAFE
-#undef GDK_MULTIHEAD_SAFE
-#endif
-
-#include <stdarg.h>
-#include <string.h>
-#include <stdio.h>
-#include <glib.h>
-#include <gtk/gtk.h>
-#include <gdk/gdk.h>
-#include <gdk/gdkx.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xmd.h>
-#include <libxfce4util/debug.h>
-#include <libxfce4util/i18n.h>
-#include <libxfce4util/utf8.h>
-#include <libxfcegui4/xinerama.h>
-#include <stdlib.h>
-#include <stdio.h>
+#include <glib.h>
+#include <gdk/gdk.h>
+#include <gdk/gdkx.h>
+#include <gtk/gtk.h>
+#include <stdarg.h>
 #include <string.h>
-#include "main.h"
+#include <stdio.h>
+#include <stdlib.h>
+#include <libxfce4util/libxfce4util.h> 
 #include "hints.h"
+#include "client.h"
 
 Atom gnome_panel_desktop_area;
 Atom motif_wm_hints;
@@ -128,6 +119,7 @@ Atom utf8_string;
 /* KDE extension */
 Atom kde_net_wm_context_help;
 Atom kde_net_wm_system_tray_window_for;
+Atom kwm_win_icon;
 
 /* Systray similation for older KDE apps */
 Atom net_system_tray_manager;
@@ -323,6 +315,7 @@ initKDEHints (Display * dpy)
         XInternAtom (dpy, "_NET_WM_CONTEXT_HELP", FALSE);
     kde_net_wm_system_tray_window_for = 
         XInternAtom (dpy, "_KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR", FALSE);
+    kwm_win_icon = XInternAtom (dpy, "KWM_WIN_ICON", FALSE);
 }
 
 void
@@ -410,8 +403,7 @@ setGnomeProtocols (Display * dpy, int screen, Window w)
     XChangeProperty (dpy, RootWindow (dpy, screen), win_protocols, XA_ATOM,
         32, PropModeReplace, (unsigned char *) atoms, 1);
     setHint (dpy, w, win_supporting_wm_check, w);
-    setHint (dpy, RootWindow (dpy, screen), win_supporting_wm_check,
-        gnome_win);
+    setHint (dpy, RootWindow (dpy, screen), win_supporting_wm_check, w);
 }
 
 void
@@ -638,7 +630,7 @@ getCardinalList (Display * dpy, Window w, Atom xatom,
 }
 
 void
-setNetWorkarea (Display * dpy, int screen, int nb_workspaces, int * m)
+setNetWorkarea (Display * dpy, int screen, int nb_workspaces, int width, int height, int * m)
 {
     unsigned long *data, *ptr;
     int i, j;
@@ -651,12 +643,8 @@ setNetWorkarea (Display * dpy, int screen, int nb_workspaces, int * m)
     {
         *ptr++ = (unsigned long) m[LEFT];
         *ptr++ = (unsigned long) m[TOP];
-        *ptr++ = (unsigned long) 
-            (MyDisplayFullWidth (dpy, screen) - 
-                (m[LEFT] + m[RIGHT]));
-        *ptr++ = (unsigned long) 
-            (MyDisplayFullHeight (dpy, screen) - 
-                (m[TOP] + m[BOTTOM]));
+        *ptr++ = (unsigned long) (width  - (m[LEFT] + m[RIGHT]));
+        *ptr++ = (unsigned long) (height - (m[TOP] + m[BOTTOM]));
     }
     XChangeProperty (dpy, RootWindow (dpy, screen), net_workarea, XA_CARDINAL,
         32, PropModeReplace, (unsigned char *) data, j * 4);
@@ -664,12 +652,12 @@ setNetWorkarea (Display * dpy, int screen, int nb_workspaces, int * m)
 }
 
 void
-initNetDesktopParams (Display * dpy, int screen, int workspace)
+initNetDesktopParams (Display * dpy, int screen, int workspace, int width, int height)
 {
     unsigned long data[2];
     TRACE ("entering initNetDesktopParams");
-    data[0] = MyDisplayFullWidth (dpy, screen);
-    data[1] = MyDisplayFullHeight (dpy, screen);
+    data[0] = width;
+    data[1] = height;
     XChangeProperty (dpy, RootWindow (dpy, screen), net_desktop_geometry,
         XA_CARDINAL, 32, PropModeReplace, (unsigned char *) data, 2);
     data[0] = 0;
@@ -898,7 +886,7 @@ sendSystrayReqDock(Display * dpy, Window window, Window systray)
     xev.window = systray;
     xev.message_type = net_system_tray_opcode;
     xev.format = 32;
-    xev.data.l[0] = CurrentTime;
+    xev.data.l[0] = GDK_CURRENT_TIME;
     xev.data.l[1] = 0; /* SYSTEM_TRAY_REQUEST_DOCK */
     xev.data.l[2] = window;
     xev.data.l[3] = 0; /* Nada */
@@ -936,12 +924,15 @@ getWindowRole (Display * dpy, Window window, char **role)
 
     if (XGetTextProperty (dpy, window, &tp, wm_window_role))
     {
-        if ((tp.value) && (tp.encoding == XA_STRING) && (tp.format == 8)
-            && (tp.nitems != 0))
+        if (tp.value)
         {
-            *role = strdup ((char *) tp.value);
+            if ((tp.encoding == XA_STRING) && (tp.format == 8) && (tp.nitems != 0))
+            {
+                *role = strdup ((char *) tp.value);
+                XFree (tp.value);
+                return TRUE;
+            }
             XFree (tp.value);
-            return TRUE;
         }
     }
 
@@ -1055,6 +1046,65 @@ getWindowCommand (Display * dpy, Window window, char ***argv, int *argc)
     return FALSE;
 }
 
+gboolean
+getKDEIcon (Display * dpy, Window window, Pixmap * pixmap, Pixmap * mask)
+{
+    Atom type;
+    int format;
+    unsigned long nitems;
+    unsigned long bytes_after;
+    Pixmap *icons;
+
+    *pixmap = None;
+    *mask = None;
+
+    icons = NULL;
+    if (XGetWindowProperty (dpy, window, kwm_win_icon, 0L, G_MAXLONG,
+            FALSE, kwm_win_icon, &type, &format, &nitems, &bytes_after,
+            (unsigned char **)&icons) != Success)
+    {
+        return FALSE;
+    }
+
+    if (type != kwm_win_icon)
+    {
+        XFree (icons);
+        return FALSE;
+    }
+
+    *pixmap = icons[0];
+    *mask = icons[1];
+
+    XFree (icons);
+
+    return TRUE;
+}
+
+gboolean
+getRGBIconData (Display * dpy, Window window, unsigned long **data, unsigned long *nitems)
+{
+    Atom type;
+    int format;
+    unsigned long bytes_after;
+
+    if (XGetWindowProperty (dpy, window, net_wm_icon, 0L, G_MAXLONG,
+                            FALSE, XA_CARDINAL, &type, &format, nitems,
+                            &bytes_after, (unsigned char **)data) != Success)
+    {
+        data = NULL;
+        return FALSE;
+    }
+    
+    if (type != XA_CARDINAL)
+    {
+        XFree (*data);
+        data = NULL;
+        return FALSE;
+    }
+
+    return TRUE;
+}
+
 #ifdef HAVE_LIBSTARTUP_NOTIFICATION
 gboolean
 getWindowStartupId (Display * dpy, Window window, char **startup_id)
@@ -1069,12 +1119,15 @@ getWindowStartupId (Display * dpy, Window window, char **startup_id)
 
     if (XGetTextProperty (dpy, window, &tp, net_startup_id))
     {
-        if ((tp.value) && (tp.encoding == XA_STRING) && (tp.format == 8)
-            && (tp.nitems != 0))
+        if (tp.value)
         {
-            *startup_id = strdup ((char *) tp.value);
+            if ((tp.encoding == XA_STRING) && (tp.format == 8) && (tp.nitems != 0))
+            {
+                *startup_id = g_strdup ((char *) tp.value);
+                XFree (tp.value);
+                return TRUE;
+            }
             XFree (tp.value);
-            return TRUE;
         }
     }
 
