@@ -1,8 +1,8 @@
 /*
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
-        the Free Software Foundation; You may only use version 2 of the License,
-        you have no option to use any other version.
+        the Free Software Foundation; either version 2, or (at your option)
+        any later version.
  
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,6 +22,14 @@
 #include <config.h>
 #endif
 
+#include <X11/Xlib.h>
+#include <glib.h>
+#include <gdk/gdk.h>
+#include <gdk/gdkx.h>
+#include <gtk/gtk.h>
+#include <libxfce4util/libxfce4util.h> 
+#include <libxfcegui4/libxfcegui4.h>
+
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <sys/types.h>
@@ -33,17 +41,7 @@
 #include <signal.h>
 #include <string.h>
 
-#include <gtk/gtk.h>
-#include <gdk/gdk.h>
-#include <gdk/gdkx.h>
-#include <glib.h>
-#include <X11/Xlib.h>
-#include <libxfce4util/debug.h>
-#include <libxfce4util/i18n.h>
-#include <libxfce4util/util.h>
-#include <libxfcegui4/libxfcegui4.h>
-
-#include "main.h"
+#include "screen.h"
 #include "events.h"
 #include "frame.h"
 #include "settings.h"
@@ -52,6 +50,7 @@
 #include "focus.h"
 #include "keyboard.h"
 #include "workspaces.h"
+#include "mywindow.h"
 #include "session.h"
 #include "startup_notification.h"
 #include "spinning_cursor.h"
@@ -66,69 +65,43 @@
     PropertyChangeMask|\
     ColormapNotify
 
-char *progname;
-Display *dpy;
-Window root, gnome_win, systray, sidewalk[2];
-Colormap cmap;
-Screen *xscreen;
-int screen;
-int depth;
-int workspace;
-gboolean use_xinerama;
-int xinerama_heads;
-int margins[4];
-int gnome_margins[4];
-int quit = FALSE, reload = FALSE;
-int shape, shape_event;
-Cursor resize_cursor[7], move_cursor, busy_cursor, root_cursor;
-SessionClient *client_session;
+static ScreenData *md = NULL;
+gboolean xfwm4_quit   = FALSE;
+gboolean xfwm4_reload = FALSE;
 
 static int
 handleXError (Display * dpy, XErrorEvent * err)
 {
 #if DEBUG            
     char buf[64];
+
+    XGetErrorText (dpy, err->error_code, buf, 63);
+    g_print ("XError: %s\n", buf);                                                  
+    g_print ("==>  XID Ox%lx, Request %d, Error %d <==\n", 
+              err->resourceid, err->request_code, err->error_code); 
 #endif
-    switch (err->error_code)
-    {
-        case BadAccess:
-            if (err->resourceid == root)
-            {
-                g_message ("%s: Another window manager is running\n",
-                    progname);
-                exit (1);
-            }
-            break;
-        default:
-#if DEBUG            
-            XGetErrorText (dpy, err->error_code, buf, 63);
-            g_print ("XError: %s\n", buf);                                                  
-            g_print ("==>  XID Ox%lx, Request %d, Error %d <==\n", 
-                      err->resourceid, err->request_code, err->error_code); 
-#endif
-            break;
-    }
     return 0;
 }
 
 static void
-cleanUp ()
+cleanUp (void)
 {
     int i;
-
+    
+    g_return_if_fail (md);
+    
     TRACE ("entering cleanUp");
 
-    clientUnframeAll ();
+    clientUnframeAll (md);
     sn_close_display ();
-    unloadSettings ();
-    XFreeCursor (dpy, root_cursor);
-    XFreeCursor (dpy, move_cursor);
-    XFreeCursor (dpy, busy_cursor);
-    xineramaFree ();
+    unloadSettings (md);
+    XFreeCursor (md->dpy, md->root_cursor);
+    XFreeCursor (md->dpy, md->move_cursor);
+    XFreeCursor (md->dpy, md->busy_cursor);
     sessionFreeWindowStates ();
     for (i = 0; i < 7; i++)
     {
-        XFreeCursor (dpy, resize_cursor[i]);
+        XFreeCursor (md->dpy, md->resize_cursor[i]);
     }
     for (i = 0; i < NB_KEY_SHORTCUTS; i++)
     {
@@ -140,10 +113,12 @@ cleanUp ()
     }
     g_free (params.workspace_names);
     params.workspace_names = NULL;
-    removeTmpEventWin (sidewalk[0]);
-    removeTmpEventWin (sidewalk[1]);
-    XSetInputFocus (dpy, root, RevertToPointerRoot, CurrentTime);
-    closeEventFilter ();
+    myWindowDelete (&md->sidewalk[0]);
+    myWindowDelete (&md->sidewalk[1]);
+    XSetInputFocus (md->dpy, md->xroot, RevertToPointerRoot, GDK_CURRENT_TIME);
+    xfce_close_event_filter (md->xfilter);
+    g_free (md);
+    md = NULL;
 }
 
 static char *build_session_filename(SessionClient *client_session)
@@ -207,7 +182,7 @@ static void
 session_die (gpointer client_data)
 {
     gtk_main_quit ();
-    quit = TRUE;
+    xfwm4_quit = TRUE;
 }
 
 static void
@@ -220,11 +195,11 @@ handleSignal (int sig)
         case SIGINT:
         case SIGTERM:
             gtk_main_quit ();
-            quit = TRUE;
+            xfwm4_quit = TRUE;
             break;
         case SIGHUP:
         case SIGUSR1:
-            reload = TRUE;
+            xfwm4_reload = TRUE;
             break;
         case SIGSEGV:
             cleanUp ();
@@ -242,49 +217,145 @@ initialize (int argc, char **argv)
     struct sigaction act;
     int dummy;
     long ws;
+    GdkWindow *groot;
+    SessionClient *client_session;
 
     TRACE ("entering initialize");
 
-    progname = argv[0];
-
-#if 0
-#ifdef ENABLE_NLS
-    bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
-#ifdef HAVE_BIND_TEXTDOMAIN_CODESET
-    bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-#endif
-    textdomain (GETTEXT_PACKAGE);
-#endif
-#else
+    md = g_new0 (ScreenData, 1);
+    
     xfce_textdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
-#endif
 
     gtk_set_locale ();
     gtk_init (&argc, &argv);
-    gdk_rgb_init();
 
-    gtk_widget_push_visual(gdk_rgb_get_visual ());
-    gtk_widget_push_colormap(gdk_rgb_get_cmap ());
-    
     DBG ("xfwm4 starting, using GTK+-%d.%d.%d", gtk_major_version, 
          gtk_minor_version, gtk_micro_version);
 
-    dpy = GDK_DISPLAY ();
-    root = GDK_ROOT_WINDOW ();
-    xscreen = DefaultScreenOfDisplay(dpy);
-    screen = XDefaultScreen (dpy);
-    depth = DefaultDepth (dpy, screen);
-    cmap = GDK_COLORMAP_XCOLORMAP(gdk_rgb_get_cmap ());
-    sn_init_display (dpy, screen);
-    workspace = 0;
+    md->gscr = gdk_screen_get_default ();
+    if (!md->gscr)
+    {
+        g_error (_("Cannot get default screen"));
+    }
+    md->gdisplay = gdk_screen_get_display (md->gscr);
+    gtk_widget_push_colormap(gdk_screen_get_rgb_colormap (md->gscr));
+    md->dpy = gdk_x11_display_get_xdisplay (md->gdisplay);
+    md->xscreen = gdk_x11_screen_get_xscreen (md->gscr);
+    groot = gdk_screen_get_root_window (md->gscr);
+    md->xroot = (Window) gdk_x11_drawable_get_xid (groot);
+    md->screen = gdk_screen_get_number (md->gscr);
+    md->cmap = GDK_COLORMAP_XCOLORMAP(gdk_screen_get_rgb_colormap (md->gscr));
+    
+    xfce_setenv ("DISPLAY", gdk_display_get_name (md->gdisplay), TRUE);
+
+    md->depth = DefaultDepth (md->dpy, md->screen);
+    sn_init_display (md);
+    md->current_ws = 0;
 
     XSetErrorHandler (handleXError);
-    shape = XShapeQueryExtension (dpy, &shape_event, &dummy);
-    use_xinerama = xineramaInit (dpy);
-    xinerama_heads = xineramaGetHeads ();
+    md->shape = XShapeQueryExtension (md->dpy, &md->shape_event, &dummy);
 
-    client_session =
-        client_session_new (argc, argv, NULL, SESSION_RESTART_IF_RUNNING, 20);
+    /* Create the side windows to detect edge movement */
+    myWindowTemp (md->dpy, md->xroot, &md->sidewalk[0], 0, 0, 
+                                  1, gdk_screen_get_height (md->gscr), 
+                                  LeaveWindowMask | PointerMotionMask);
+
+    myWindowTemp (md->dpy, md->xroot, &md->sidewalk[1], 
+                                  gdk_screen_get_width (md->gscr) - 1, 0, 
+                                  1, gdk_screen_get_height (md->gscr), 
+                                  LeaveWindowMask | PointerMotionMask);
+
+    md->margins[TOP] = md->gnome_margins[TOP] = 0;
+    md->margins[LEFT] = md->gnome_margins[LEFT] = 0;
+    md->margins[RIGHT] = md->gnome_margins[RIGHT] = 0;
+    md->margins[BOTTOM] = md->gnome_margins[BOTTOM] = 0;
+
+    initICCCMHints (md->dpy);
+    initMotifHints (md->dpy);
+    initGnomeHints (md->dpy);
+    initNetHints (md->dpy);
+    initKDEHints (md->dpy);
+    initSystrayHints (md->dpy, md->screen);
+    
+    initModifiers (md->dpy);
+
+    md->root_cursor = XCreateFontCursor (md->dpy, XC_left_ptr);
+    md->move_cursor = XCreateFontCursor (md->dpy, XC_fleur);
+    md->busy_cursor = cursorCreateSpinning (md->dpy, md->xroot);
+    md->resize_cursor[CORNER_TOP_LEFT] =
+        XCreateFontCursor (md->dpy, XC_top_left_corner);
+    md->resize_cursor[CORNER_TOP_RIGHT] =
+        XCreateFontCursor (md->dpy, XC_top_right_corner);
+    md->resize_cursor[CORNER_BOTTOM_LEFT] =
+        XCreateFontCursor (md->dpy, XC_bottom_left_corner);
+    md->resize_cursor[CORNER_BOTTOM_RIGHT] =
+        XCreateFontCursor (md->dpy, XC_bottom_right_corner);
+    md->resize_cursor[4 + SIDE_LEFT] = XCreateFontCursor (md->dpy, XC_left_side);
+    md->resize_cursor[4 + SIDE_RIGHT] = XCreateFontCursor (md->dpy, XC_right_side);
+    md->resize_cursor[4 + SIDE_BOTTOM] = XCreateFontCursor (md->dpy, XC_bottom_side);
+
+    XDefineCursor (md->dpy, md->xroot, md->root_cursor);
+
+    md->xfilter = xfce_init_event_filter (md->gscr, MAIN_EVENT_MASK, (gpointer) md, "xfwm");
+    if (!md->xfilter)
+    {
+        return -1;
+    }
+    xfce_push_event_filter (md->xfilter, xfwm4_event_filter, (gpointer) md);
+
+    md->gnome_win = xfce_get_default_XID (md->xfilter);
+    DBG ("Our event window is 0x%lx", md->gnome_win);
+
+    if (!initSettings (md))
+    {
+        return -2;
+    }
+
+    md->systray = getSystrayWindow (md->dpy);
+    setGnomeProtocols (md->dpy, md->screen, md->gnome_win);
+    setHint (md->dpy, md->xroot, win_supporting_wm_check, md->gnome_win);
+    setHint (md->dpy, md->xroot, win_desktop_button_proxy, md->gnome_win);
+    setHint (md->dpy, md->gnome_win, win_desktop_button_proxy, md->gnome_win);
+    getHint (md->dpy, md->xroot, win_workspace, &ws);
+    md->current_ws = (int) ws;
+    getGnomeDesktopMargins (md->dpy, md->screen, md->gnome_margins);
+    set_utf8_string_hint (md->dpy, md->gnome_win, net_wm_name, "Xfwm4");
+    setNetSupportedHint (md->dpy, md->screen, md->gnome_win);
+    workspaceUpdateArea (md);
+    initNetDesktopParams (md->dpy, md->screen, 
+                               md->current_ws,
+                               gdk_screen_get_width (md->gscr), 
+                               gdk_screen_get_height (md->gscr));
+    setNetWorkarea (md->dpy, md->screen, params.workspace_count, 
+                         gdk_screen_get_width (md->gscr), 
+                         gdk_screen_get_height (md->gscr), 
+                         md->margins);
+    XSetInputFocus (md->dpy, md->gnome_win, RevertToPointerRoot, GDK_CURRENT_TIME);
+    initGtkCallbacks (md);
+    initMenuEventWin ();
+    
+    /* The first time the first Gtk application on a display uses pango,
+     * pango grabs the XServer while it creates the font cache window.
+     * Therefore, force the cache window to be created now instead of
+     * trying to do it while we have another grab and deadlocking the server.
+     */
+    layout = gtk_widget_create_pango_layout (xfce_get_default_gtk_widget (md->xfilter), "-");
+    pango_layout_get_pixel_extents (layout, NULL, NULL);
+    g_object_unref (G_OBJECT (layout));
+
+    clientClearFocus ();
+    clientFrameAll (md);
+
+    act.sa_handler = handleSignal;
+    act.sa_flags = 0;
+    sigaction (SIGINT,  &act, NULL);
+    sigaction (SIGTERM, &act, NULL);
+    sigaction (SIGHUP,  &act, NULL);
+    sigaction (SIGUSR1, &act, NULL);
+    sigaction (SIGSEGV, &act, NULL);
+
+    client_session = client_session_new (argc, argv, (gpointer) md, 
+                                         SESSION_RESTART_IF_RUNNING, 20);
     client_session->data = (gpointer) client_session;
     client_session->save_phase_2 = save_phase_2;
     client_session->die = session_die;
@@ -294,96 +365,6 @@ initialize (int argc, char **argv)
         load_saved_session (client_session);
     }
 
-    /* Create the side windows to detect edge movement */
-    sidewalk[0] = setTmpEventWin (0, 0, 
-                                  1, MyDisplayFullHeight (dpy, screen), 
-                                  LeaveWindowMask | PointerMotionMask);
-
-    sidewalk[1] = setTmpEventWin (MyDisplayFullWidth (dpy, screen) - 1, 0, 
-                                  1, MyDisplayFullHeight (dpy, screen), 
-                                  LeaveWindowMask | PointerMotionMask);
-
-    margins[TOP] = gnome_margins[TOP] = 0;
-    margins[LEFT] = gnome_margins[LEFT] = 0;
-    margins[RIGHT] = gnome_margins[RIGHT] = 0;
-    margins[BOTTOM] = gnome_margins[BOTTOM] = 0;
-
-    initICCCMHints (dpy);
-    initMotifHints (dpy);
-    initGnomeHints (dpy);
-    initNetHints (dpy);
-    initKDEHints (dpy);
-    initSystrayHints (dpy, screen);
-    
-    initModifiers (dpy);
-
-    root_cursor = XCreateFontCursor (dpy, XC_left_ptr);
-    move_cursor = XCreateFontCursor (dpy, XC_fleur);
-    busy_cursor = cursorCreateSpinning (dpy, root);
-    resize_cursor[CORNER_TOP_LEFT] =
-        XCreateFontCursor (dpy, XC_top_left_corner);
-    resize_cursor[CORNER_TOP_RIGHT] =
-        XCreateFontCursor (dpy, XC_top_right_corner);
-    resize_cursor[CORNER_BOTTOM_LEFT] =
-        XCreateFontCursor (dpy, XC_bottom_left_corner);
-    resize_cursor[CORNER_BOTTOM_RIGHT] =
-        XCreateFontCursor (dpy, XC_bottom_right_corner);
-    resize_cursor[4 + SIDE_LEFT] = XCreateFontCursor (dpy, XC_left_side);
-    resize_cursor[4 + SIDE_RIGHT] = XCreateFontCursor (dpy, XC_right_side);
-    resize_cursor[4 + SIDE_BOTTOM] = XCreateFontCursor (dpy, XC_bottom_side);
-
-    XDefineCursor (dpy, root, root_cursor);
-
-    if (!initEventFilter (MAIN_EVENT_MASK, NULL, "xfwm"))
-    {
-        return -1;
-    }
-    pushEventFilter (xfwm4_event_filter, NULL);
-
-    gnome_win = getDefaultXWindow ();
-    DBG ("Our event window is 0x%lx", gnome_win);
-
-    if (!initSettings ())
-    {
-        return -2;
-    }
-
-    systray = getSystrayWindow (dpy);
-    setGnomeProtocols (dpy, screen, gnome_win);
-    setHint (dpy, root, win_supporting_wm_check, gnome_win);
-    setHint (dpy, root, win_desktop_button_proxy, gnome_win);
-    setHint (dpy, gnome_win, win_desktop_button_proxy, gnome_win);
-    getHint (dpy, root, win_workspace, &ws);
-    workspace = (int) ws;
-    getGnomeDesktopMargins (dpy, screen, gnome_margins);
-    set_utf8_string_hint (dpy, gnome_win, net_wm_name, "Xfwm4");
-    setNetSupportedHint (dpy, screen, gnome_win);
-    workspaceUpdateArea (margins, gnome_margins);
-    initNetDesktopParams (dpy, screen, workspace);
-    setNetWorkarea (dpy, screen, params.workspace_count, margins);
-    XSetInputFocus (dpy, gnome_win, RevertToPointerRoot, CurrentTime);
-    initGtkCallbacks ();
-    
-    /* The first time the first Gtk application on a display uses pango,
-     * pango grabs the XServer while it creates the font cache window.
-     * Therefore, force the cache window to be created now instead of
-     * trying to do it while we have another grab and deadlocking the server.
-     */
-    layout = gtk_widget_create_pango_layout (getDefaultGtkWidget (), "-");
-    pango_layout_get_pixel_extents (layout, NULL, NULL);
-    g_object_unref (G_OBJECT (layout));
-
-    clientClearFocus ();
-    clientFrameAll ();
-
-    act.sa_handler = handleSignal;
-    act.sa_flags = 0;
-    sigaction (SIGINT, &act, NULL);
-    sigaction (SIGTERM, &act, NULL);
-    sigaction (SIGHUP, &act, NULL);
-    sigaction (SIGUSR1, &act, NULL);
-    sigaction (SIGSEGV, &act, NULL);
-
     return 0;
 }
 
