@@ -1,8 +1,8 @@
 /*
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
-        the Free Software Foundation; You may only use version 2 of the License,
-        you have no option to use any other version.
+        the Free Software Foundation; either version 2, or (at your option)
+        any later version.
  
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,7 +14,7 @@
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2003 Olivier Fourdan
+        xfwm4    - (c) 2002-2004 Olivier Fourdan
  
  */
 
@@ -28,16 +28,15 @@
 #include <sys/types.h>
 #include <X11/Xlib.h>
 #include <glib.h>
+#include "screen.h"
 
-void getMouseXY (Window, int *, int *);
-Window getMouseWindow (Window);
-GC createGC (Colormap, char *, int, XFontStruct *, int, gboolean);
-void sendClientMessage (Window, Atom, Atom, Time);
-void myXGrabServer (void);
-void myXUngrabServer (void);
-gboolean myCheckWindow(Window);
-Window setTmpEventWin (int, int, unsigned int, unsigned int, long);
-void removeTmpEventWin (Window);
-void placeSidewalks(gboolean);
+void getMouseXY (ScreenData *, Window, int *, int *);
+Window getMouseWindow (ScreenData *, Window);
+GC createGC (ScreenData *, char *, int, XFontStruct *, int, gboolean);
+void sendClientMessage (ScreenData *, Window, Atom, Atom, Time);
+void myXGrabServer (ScreenData *);
+void myXUngrabServer (ScreenData *);
+gboolean myCheckWindow(ScreenData *, Window);
+void placeSidewalks(ScreenData *, gboolean);
 
 #endif /* INC_MISC_H */
