@@ -1,8 +1,8 @@
 /*
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
-        the Free Software Foundation; You may only use version 2 of the License,
-        you have no option to use any other version.
+        the Free Software Foundation; either version 2, or (at your option)
+        any later version.
  
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,15 +24,14 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xatom.h>
-#include <gdk/gdk.h>
-#include <gtk/gtk.h>
-#include <gdk/gdkx.h>
 #include <glib.h>
-#include <libxfce4util/debug.h>
-#include <libxfce4util/i18n.h>
+#include <gdk/gdk.h>
+#include <gdk/gdkx.h>
+#include <gtk/gtk.h>
+#include <libxfce4util/libxfce4util.h> 
 #include <libxfcegui4/libxfcegui4.h>
 
-#include "main.h"
+#include "screen.h"
 #include "netwm.h"
 #include "misc.h"
 #include "client.h"
@@ -114,13 +113,13 @@ clientSetNetState (Client * c)
         TRACE ("clientSetNetState : hidden");
         data[i++] = net_wm_state_hidden;
     }
-    XChangeProperty (dpy, c->window, net_wm_state, XA_ATOM, 32,
+    XChangeProperty (c->md->dpy, c->window, net_wm_state, XA_ATOM, 32,
         PropModeReplace, (unsigned char *) data, i);
     /*
        We also set GNOME hint here for consistency and convenience, 
        although the meaning of net_wm_state and win_state aren't the same.
      */
-    setHint (dpy, c->window, win_state, c->win_state);
+    setHint (c->md->dpy, c->window, win_state, c->win_state);
 }
 
 void
@@ -133,7 +132,7 @@ clientGetNetState (Client * c)
     TRACE ("entering clientGetNetState");
     TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
 
-    if (getAtomList (dpy, c->window, net_wm_state, &atoms, &n_atoms))
+    if (getAtomList (c->md->dpy, c->window, net_wm_state, &atoms, &n_atoms))
     {
         int i;
         TRACE ("clientGetNetState: %i atoms detected", n_atoms);
@@ -504,21 +503,26 @@ clientUpdateFullscreenState (Client * c)
 
     if (FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
     {
+        GdkRectangle rect;
+        gint monitor_nbr;
         int cx, cy;
 
         cx = frameX (c) + (frameWidth (c) / 2);
         cy = frameY (c) + (frameHeight (c) / 2);
 
+        monitor_nbr = gdk_screen_get_monitor_at_point (c->md->gscr, cx, cy);
+        gdk_screen_get_monitor_geometry (c->md->gscr, monitor_nbr, &rect);
+
         c->fullscreen_old_x = c->x;
         c->fullscreen_old_y = c->y;
         c->fullscreen_old_width = c->width;
         c->fullscreen_old_height = c->height;
         c->fullscreen_old_layer = c->win_layer;
 
-        wc.x = MyDisplayX (cx, cy);
-        wc.y = MyDisplayY (cx, cy);
-        wc.width = MyDisplayWidth (dpy, screen, cx, cy);
-        wc.height = MyDisplayHeight (dpy, screen, cx, cy);
+        wc.x = rect.x;
+        wc.y = rect.y;
+        wc.width = rect.width;
+        wc.height = rect.height;
         layer = WIN_LAYER_ABOVE_DOCK;
     }
     else
@@ -538,10 +542,10 @@ clientUpdateFullscreenState (Client * c)
            grab focus in focus follow mouse mode. Grab the pointer to
            avoid these effects
          */
-        XGrabPointer (dpy, gnome_win, FALSE, EnterWindowMask, GrabModeAsync,
-                           GrabModeAsync, None, None, CurrentTime);
+        XGrabPointer (c->md->dpy, c->md->gnome_win, FALSE, EnterWindowMask, GrabModeAsync,
+                           GrabModeAsync, None, None, GDK_CURRENT_TIME);
         clientConfigure (c, &wc, CWX | CWY | CWWidth | CWHeight, NO_CFG_FLAG);
-        XUngrabPointer (dpy, CurrentTime);
+        XUngrabPointer (c->md->dpy, GDK_CURRENT_TIME);
     }
     else
     {
@@ -567,7 +571,7 @@ clientGetNetWmType (Client * c)
     n_atoms = 0;
     atoms = NULL;
 
-    if (!getAtomList (dpy, c->window, net_wm_window_type, &atoms, &n_atoms))
+    if (!getAtomList (c->md->dpy, c->window, net_wm_window_type, &atoms, &n_atoms))
     {
         switch (c->win_layer)
         {
@@ -640,9 +644,9 @@ clientGetInitialNetWmDesktop (Client * c)
                 CLIENT_FLAG_SESSION_MANAGED | CLIENT_FLAG_WORKSPACE_SET))
         {
             FLAG_SET (c->flags, CLIENT_FLAG_WORKSPACE_SET);
-            c->win_workspace = workspace;
+            c->win_workspace = c->md->current_ws;
         }
-        if (getHint (dpy, c->window, net_wm_desktop, &val))
+        if (getHint (c->md->dpy, c->window, net_wm_desktop, &val))
         {
             TRACE ("atom net_wm_desktop detected");
             if (val == (int) ALL_WORKSPACES)
@@ -656,7 +660,7 @@ clientGetInitialNetWmDesktop (Client * c)
                     FLAG_SET (c->flags, CLIENT_FLAG_STICKY);
                     c->win_state |= WIN_STATE_STICKY;
                 }
-                c->win_workspace = workspace;
+                c->win_workspace = c->md->current_ws;
             }
             else
             {
@@ -667,7 +671,7 @@ clientGetInitialNetWmDesktop (Client * c)
             }
             FLAG_SET (c->flags, CLIENT_FLAG_WORKSPACE_SET);
         }
-        else if (getHint (dpy, c->window, win_workspace, &val))
+        else if (getHint (c->md->dpy, c->window, win_workspace, &val))
         {
             TRACE ("atom win_workspace specifies window \"%s\" is on desk %i",
                 c->name, (int) val);
@@ -686,21 +690,19 @@ clientGetInitialNetWmDesktop (Client * c)
     }
     TRACE ("initial desktop for window \"%s\" is %i", c->name,
         c->win_workspace);
-    setHint (dpy, c->window, win_workspace, c->win_workspace);
+    setHint (c->md->dpy, c->window, win_workspace, c->win_workspace);
     if (FLAG_TEST (c->flags, CLIENT_FLAG_STICKY))
     {
-        setHint (dpy, c->window, net_wm_desktop,
-            (unsigned long) ALL_WORKSPACES);
+        setHint (c->md->dpy, c->window, net_wm_desktop, (unsigned long) ALL_WORKSPACES);
     }
     else
     {
-        setHint (dpy, c->window, net_wm_desktop,
-            (unsigned long) c->win_workspace);
+        setHint (c->md->dpy, c->window, net_wm_desktop, (unsigned long) c->win_workspace);
     }
 }
 
 void
-clientSetNetClientList (Atom a, GList * list)
+clientSetNetClientList (ScreenData * md, Atom a, GList * list)
 {
     Window *listw;
     Window *index_dest;
@@ -712,7 +714,7 @@ clientSetNetClientList (Atom a, GList * list)
     size = g_list_length (list);
     if (size < 1)
     {
-        XDeleteProperty (dpy, root, a);
+        XDeleteProperty (md->dpy, md->xroot, a);
     }
     else if ((listw = (Window *) malloc ((size + 1) * sizeof (Window))))
     {
@@ -723,7 +725,7 @@ clientSetNetClientList (Atom a, GList * list)
             Client *c = (Client *) index_src->data;
             *index_dest = c->window;
         }
-        XChangeProperty (dpy, root, a, XA_WINDOW, 32, PropModeReplace,
+        XChangeProperty (md->dpy, md->xroot, a, XA_WINDOW, 32, PropModeReplace,
             (unsigned char *) listw, size);
         free (listw);
     }
@@ -747,7 +749,7 @@ clientGetNetStruts (Client * c)
     FLAG_UNSET (c->flags, CLIENT_FLAG_HAS_STRUT);
     FLAG_UNSET (c->flags, CLIENT_FLAG_HAS_STRUT_PARTIAL);
 
-    if (getCardinalList (dpy, c->window, net_wm_strut_partial, &struts, &nitems))
+    if (getCardinalList (c->md->dpy, c->window, net_wm_strut_partial, &struts, &nitems))
     {
         if (nitems != 12)
         {
@@ -763,9 +765,9 @@ clientGetNetStruts (Client * c)
         }
 
         XFree (struts);
-        workspaceUpdateArea (margins, gnome_margins);
+        workspaceUpdateArea (c->md);
     }
-    else if (getCardinalList (dpy, c->window, net_wm_strut, &struts, &nitems))
+    else if (getCardinalList (c->md->dpy, c->window, net_wm_strut, &struts, &nitems))
     {
         if (nitems != 4)
         {
@@ -778,20 +780,20 @@ clientGetNetStruts (Client * c)
         {
             c->struts[i] = (int) struts[i];
         }
-	for (i = 4; i < 12; i++)
-	{
+        for (i = 4; i < 12; i++)
+        {
             c->struts[i] = 0;
-	}
-	/* Fill(in values as for partial struts */
+        }
+        /* Fill(in values as for partial struts */
         c->struts[TOP_START_X] = c->struts[BOTTOM_START_X] = 0;
         c->struts[TOP_END_X] = c->struts[BOTTOM_END_X] = 
-	    MyDisplayFullWidth (dpy, screen);
+            gdk_screen_get_width (c->md->gscr);
         c->struts[LEFT_START_Y] = c->struts[RIGHT_START_Y] = 0;
         c->struts[LEFT_END_Y] = c->struts[RIGHT_END_Y] = 
-	    MyDisplayFullHeight (dpy, screen);
-	
+            gdk_screen_get_height (c->md->gscr);
+        
         XFree (struts);
-        workspaceUpdateArea (margins, gnome_margins);
+        workspaceUpdateArea (c->md);
     }
 }
 
@@ -816,7 +818,7 @@ clientSetNetActions (Client * c)
     {
         atoms[i++] = net_wm_action_shade;
     }
-    XChangeProperty (dpy, c->window, net_wm_allowed_actions, XA_ATOM, 32,
+    XChangeProperty (c->md->dpy, c->window, net_wm_allowed_actions, XA_ATOM, 32,
         PropModeReplace, (unsigned char *) atoms, i);
 }
 
