@@ -1,8 +1,8 @@
 /*
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
-        the Free Software Foundation; You may only use version 2 of the License,
-        you have no option to use any other version.
+        the Free Software Foundation; either version 2, or (at your option)
+        any later version.
  
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,7 +13,7 @@
         along with this program; if not, write to the Free Software
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
-        xfwm4    - (c) 2004 Olivier Fourdan
+        xfwm4    - (c) 2002-2004 Olivier Fourdan
           based on a patch from Joshua Blanton <jblanton@irg.cs.ohiou.edu>
  */
 
@@ -37,7 +37,7 @@ struct _Poswin
     GtkWidget *label;
 };
 
-Poswin *poswinCreate (void);
+Poswin *poswinCreate (GdkScreen *);
 void poswinSetPosition (Poswin * poswin,  Client *c);
 void poswinDestroy (Poswin * poswin);
 void poswinShow (Poswin * poswin);
