@@ -1,8 +1,8 @@
 /*
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
-        the Free Software Foundation; You may only use version 2 of the License,
-        you have no option to use any other version.
+        the Free Software Foundation; either version 2, or (at your option)
+        any later version.
  
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,31 +14,27 @@
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2003 Olivier Fourdan
+        xfwm4    - (c) 2002-2004 Olivier Fourdan
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#ifdef GDK_MULTIHEAD_SAFE
-#undef GDK_MULTIHEAD_SAFE
-#endif
-
 #include <glib.h>
 #include <pango/pango.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
-#include <libxfce4util/debug.h>
-#include <libxfce4util/i18n.h>
-#include <libxfce4util/util.h>
+#include <libxfce4util/libxfce4util.h> 
 #include <libxfcegui4/libxfcegui4.h>
 #include <libxfce4mcs/mcs-client.h>
-#include "main.h"
+
+#include "screen.h"
 #include "hints.h"
 #include "parserc.h"
 #include "client.h"
+#include "focus.h"
 #include "workspaces.h"
 
 #define CHANNEL1 "xfwm4"
@@ -57,7 +53,7 @@ static McsClient *client = NULL;
 static int mcs_initted = FALSE;
 
 static void
-set_settings_margin (int idx, int value)
+set_settings_margin (ScreenData *md, int idx, int value)
 {
     int val;
 
@@ -69,9 +65,9 @@ set_settings_margin (int idx, int value)
             {
                 val = 0;
             }
-            else if (value > MyDisplayFullWidth (dpy, screen) / 4)
+            else if (value > gdk_screen_get_width (md->gscr) / 4)
             {
-                val = MyDisplayFullWidth (dpy, screen) / 4;
+                val = gdk_screen_get_width (md->gscr) / 4;
             }
             else
             {
@@ -85,9 +81,9 @@ set_settings_margin (int idx, int value)
             {
                 val = 0;
             }
-            else if (value > MyDisplayFullHeight (dpy, screen) / 4)
+            else if (value > gdk_screen_get_height (md->gscr) / 4)
             {
-                val = MyDisplayFullHeight (dpy, screen) / 4;
+                val = gdk_screen_get_height (md->gscr) / 4;
             }
             else
             {
@@ -104,12 +100,16 @@ static void
 notify_cb (const char *name, const char *channel_name, McsAction action,
     McsSetting * setting, void *data)
 {
+    ScreenData *md = (ScreenData *) data;
+    
+    g_return_if_fail (md != NULL);
+
     if (!g_ascii_strcasecmp (CHANNEL1, channel_name))
     {
         switch (action)
         {
             case MCS_ACTION_NEW:
-                /* The following is to reduce initial startup time and reloads */
+                /* The following is to reduce initial startup time and reload */
                 if (!mcs_initted)
                 {
                     return;
@@ -153,7 +153,7 @@ notify_cb (const char *name, const char *channel_name, McsAction action,
                     else if (!strcmp (name, "Xfwm/WrapWorkspaces"))
                     {
                         params.wrap_workspaces = setting->data.v_int;
-                        placeSidewalks (params.wrap_workspaces);
+                        placeSidewalks (md, params.wrap_workspaces);
                     }
                     else if (!strcmp (name, "Xfwm/WrapWindows"))
                     {
@@ -176,27 +176,27 @@ notify_cb (const char *name, const char *channel_name, McsAction action,
                 {
                     if (!strcmp (name, "Xfwm/DblClickAction"))
                     {
-                        reloadSettings (NO_UPDATE_FLAG);
+                        reloadSettings (md, NO_UPDATE_FLAG);
                     }
                     else if (!strcmp (name, "Xfwm/KeyThemeName"))
                     {
-                        reloadSettings (UPDATE_KEYGRABS);
+                        reloadSettings (md, UPDATE_KEYGRABS);
                     }
                     else if (!strcmp (name, "Xfwm/ThemeName"))
                     {
-                        reloadSettings (UPDATE_GRAVITY | UPDATE_CACHE);
+                        reloadSettings (md, UPDATE_GRAVITY | UPDATE_CACHE);
                     }
                     else if (!strcmp (name, "Xfwm/ButtonLayout"))
                     {
-                        reloadSettings (UPDATE_FRAME | UPDATE_CACHE);
+                        reloadSettings (md, UPDATE_FRAME | UPDATE_CACHE);
                     }
                     if (!strcmp (name, "Xfwm/TitleAlign"))
                     {
-                        reloadSettings (UPDATE_FRAME | UPDATE_CACHE);
+                        reloadSettings (md, UPDATE_FRAME | UPDATE_CACHE);
                     }
                     if (!strcmp (name, "Xfwm/TitleFont"))
                     {
-                        reloadSettings (UPDATE_FRAME | UPDATE_CACHE);
+                        reloadSettings (md, UPDATE_FRAME | UPDATE_CACHE);
                     }
                 }
                 break;
@@ -220,22 +220,19 @@ notify_cb (const char *name, const char *channel_name, McsAction action,
                 {
                     if (!strcmp (name, "Xfwm/LeftMargin"))
                     {
-                        set_settings_margin (LEFT,
-                            setting->data.v_int);
+                        set_settings_margin (md, LEFT, setting->data.v_int);
                     }
                     else if (!strcmp (name, "Xfwm/RightMargin"))
                     {
-                        set_settings_margin (RIGHT,
-                            setting->data.v_int);
+                        set_settings_margin (md, RIGHT, setting->data.v_int);
                     }
                     else if (!strcmp (name, "Xfwm/BottomMargin"))
                     {
-                        set_settings_margin (BOTTOM,
-                            setting->data.v_int);
+                        set_settings_margin (md, BOTTOM, setting->data.v_int);
                     }
                     else if (!strcmp (name, "Xfwm/TopMargin"))
                     {
-                        set_settings_margin (TOP, setting->data.v_int);
+                        set_settings_margin (md, TOP, setting->data.v_int);
                     }
                 }
                 break;
@@ -259,7 +256,7 @@ notify_cb (const char *name, const char *channel_name, McsAction action,
                 {
                     if (!strcmp (name, "Xfwm/WorkspaceCount"))
                     {
-                        workspaceSetCount(setting->data.v_int);
+                        workspaceSetCount(md, setting->data.v_int);
                     }
                 }
                 break;
@@ -307,7 +304,7 @@ watch_cb (Window window, Bool is_start, long mask, void *cb_data)
 }
 
 static void
-loadRcData (Settings rc[])
+loadRcData (ScreenData *md, Settings rc[])
 {
     const gchar *homedir = xfce_get_userdir ();
     gchar *keytheme;
@@ -315,7 +312,7 @@ loadRcData (Settings rc[])
 
     if (!parseRc ("defaults", PACKAGE_DATADIR, rc))
     {
-        g_warning (_("%s: Missing defaults file"), progname);
+        g_warning (_("%s: Missing defaults file"), g_get_prgname ());
         exit (1);
     }
     keythemevalue = getValue ("keytheme", rc);
@@ -329,7 +326,7 @@ loadRcData (Settings rc[])
 }
 
 static void
-loadMcsData (Settings rc[])
+loadMcsData (ScreenData *md, Settings rc[])
 {
     McsSetting *setting;
     if (client)
@@ -483,12 +480,12 @@ loadMcsData (Settings rc[])
 }
 
 static void
-loadTheme (Settings rc[])
+loadTheme (ScreenData *md, Settings rc[])
 {
     gchar *theme;
     gchar *font;
     XpmColorSymbol colsym[20];
-    GtkWidget *widget = getDefaultGtkWidget ();
+    GtkWidget *widget = xfce_get_default_gtk_widget (md->xfilter);
     PangoFontDescription *desc;
     guint i;
 
@@ -524,13 +521,13 @@ loadTheme (Settings rc[])
 
     if (params.title_colors[ACTIVE].allocated)
     {
-        gdk_colormap_free_colors (gdk_rgb_get_cmap (),
+        gdk_colormap_free_colors (gdk_screen_get_rgb_colormap (md->gscr),
             &params.title_colors[ACTIVE].col, 1);
         params.title_colors[ACTIVE].allocated = FALSE;
     }
     if (gdk_color_parse (rc[0].value, &params.title_colors[ACTIVE].col))
     {
-        if (gdk_colormap_alloc_color (gdk_rgb_get_cmap (),
+        if (gdk_colormap_alloc_color (gdk_screen_get_rgb_colormap (md->gscr),
                 &params.title_colors[ACTIVE].col, FALSE, FALSE))
         {
             params.title_colors[ACTIVE].allocated = TRUE;
@@ -539,7 +536,7 @@ loadTheme (Settings rc[])
                 g_object_unref (G_OBJECT (params.title_colors[ACTIVE].gc));
             }
             params.title_colors[ACTIVE].gc =
-                gdk_gc_new (getDefaultGdkWindow ());
+                gdk_gc_new (xfce_get_default_gdk_window (md->xfilter));
             gdk_gc_copy (params.title_colors[ACTIVE].gc, get_style_gc (widget,
                     "text", "selected"));
             gdk_gc_set_foreground (params.title_colors[ACTIVE].gc,
@@ -575,13 +572,13 @@ loadTheme (Settings rc[])
 
     if (params.title_colors[INACTIVE].allocated)
     {
-        gdk_colormap_free_colors (gdk_rgb_get_cmap (),
+        gdk_colormap_free_colors (gdk_screen_get_rgb_colormap (md->gscr),
             &params.title_colors[INACTIVE].col, 1);
         params.title_colors[INACTIVE].allocated = FALSE;
     }
     if (gdk_color_parse (rc[1].value, &params.title_colors[INACTIVE].col))
     {
-        if (gdk_colormap_alloc_color (gdk_rgb_get_cmap (),
+        if (gdk_colormap_alloc_color (gdk_screen_get_rgb_colormap (md->gscr),
                 &params.title_colors[INACTIVE].col, FALSE, FALSE))
         {
             params.title_colors[INACTIVE].allocated = TRUE;
@@ -590,7 +587,7 @@ loadTheme (Settings rc[])
                 g_object_unref (G_OBJECT (params.title_colors[INACTIVE].gc));
             }
             params.title_colors[INACTIVE].gc =
-                gdk_gc_new (getDefaultGdkWindow ());
+                gdk_gc_new (xfce_get_default_gdk_window (md->xfilter));
             gdk_gc_copy (params.title_colors[INACTIVE].gc,
                 get_style_gc (widget, "text", "normal"));
             gdk_gc_set_foreground (params.title_colors[INACTIVE].gc,
@@ -617,110 +614,111 @@ loadTheme (Settings rc[])
         if (desc)
         {
             gtk_widget_modify_font (widget, desc);
+            g_free (desc);
         }
     }
 
-    myPixmapLoad (dpy, &params.sides[SIDE_LEFT][ACTIVE], theme,
+    myPixmapLoad (md, &params.sides[SIDE_LEFT][ACTIVE], theme,
         "left-active", colsym, 20);
-    myPixmapLoad (dpy, &params.sides[SIDE_LEFT][INACTIVE], theme,
+    myPixmapLoad (md, &params.sides[SIDE_LEFT][INACTIVE], theme,
         "left-inactive", colsym, 20);
-    myPixmapLoad (dpy, &params.sides[SIDE_RIGHT][ACTIVE], theme,
+    myPixmapLoad (md, &params.sides[SIDE_RIGHT][ACTIVE], theme,
         "right-active", colsym, 20);
-    myPixmapLoad (dpy, &params.sides[SIDE_RIGHT][INACTIVE], theme,
+    myPixmapLoad (md, &params.sides[SIDE_RIGHT][INACTIVE], theme,
         "right-inactive", colsym, 20);
-    myPixmapLoad (dpy, &params.sides[SIDE_BOTTOM][ACTIVE], theme,
+    myPixmapLoad (md, &params.sides[SIDE_BOTTOM][ACTIVE], theme,
         "bottom-active", colsym, 20);
-    myPixmapLoad (dpy, &params.sides[SIDE_BOTTOM][INACTIVE], theme,
+    myPixmapLoad (md, &params.sides[SIDE_BOTTOM][INACTIVE], theme,
         "bottom-inactive", colsym, 20);
-    myPixmapLoad (dpy, &params.corners[CORNER_TOP_LEFT][ACTIVE], theme,
+    myPixmapLoad (md, &params.corners[CORNER_TOP_LEFT][ACTIVE], theme,
         "top-left-active", colsym, 20);
-    myPixmapLoad (dpy, &params.corners[CORNER_TOP_LEFT][INACTIVE], theme,
+    myPixmapLoad (md, &params.corners[CORNER_TOP_LEFT][INACTIVE], theme,
         "top-left-inactive", colsym, 20);
-    myPixmapLoad (dpy, &params.corners[CORNER_TOP_RIGHT][ACTIVE], theme,
+    myPixmapLoad (md, &params.corners[CORNER_TOP_RIGHT][ACTIVE], theme,
         "top-right-active", colsym, 20);
-    myPixmapLoad (dpy, &params.corners[CORNER_TOP_RIGHT][INACTIVE], theme,
+    myPixmapLoad (md, &params.corners[CORNER_TOP_RIGHT][INACTIVE], theme,
         "top-right-inactive", colsym, 20);
-    myPixmapLoad (dpy, &params.corners[CORNER_BOTTOM_LEFT][ACTIVE], theme,
+    myPixmapLoad (md, &params.corners[CORNER_BOTTOM_LEFT][ACTIVE], theme,
         "bottom-left-active", colsym, 20);
-    myPixmapLoad (dpy, &params.corners[CORNER_BOTTOM_LEFT][INACTIVE], theme,
+    myPixmapLoad (md, &params.corners[CORNER_BOTTOM_LEFT][INACTIVE], theme,
         "bottom-left-inactive", colsym, 20);
-    myPixmapLoad (dpy, &params.corners[CORNER_BOTTOM_RIGHT][ACTIVE], theme,
+    myPixmapLoad (md, &params.corners[CORNER_BOTTOM_RIGHT][ACTIVE], theme,
         "bottom-right-active", colsym, 20);
-    myPixmapLoad (dpy, &params.corners[CORNER_BOTTOM_RIGHT][INACTIVE], theme,
+    myPixmapLoad (md, &params.corners[CORNER_BOTTOM_RIGHT][INACTIVE], theme,
         "bottom-right-inactive", colsym, 20);
-    myPixmapLoad (dpy, &params.buttons[HIDE_BUTTON][ACTIVE], theme,
+    myPixmapLoad (md, &params.buttons[HIDE_BUTTON][ACTIVE], theme,
         "hide-active", colsym, 20);
-    myPixmapLoad (dpy, &params.buttons[HIDE_BUTTON][INACTIVE], theme,
+    myPixmapLoad (md, &params.buttons[HIDE_BUTTON][INACTIVE], theme,
         "hide-inactive", colsym, 20);
-    myPixmapLoad (dpy, &params.buttons[HIDE_BUTTON][PRESSED], theme,
+    myPixmapLoad (md, &params.buttons[HIDE_BUTTON][PRESSED], theme,
         "hide-pressed", colsym, 20);
-    myPixmapLoad (dpy, &params.buttons[CLOSE_BUTTON][ACTIVE], theme,
+    myPixmapLoad (md, &params.buttons[CLOSE_BUTTON][ACTIVE], theme,
         "close-active", colsym, 20);
-    myPixmapLoad (dpy, &params.buttons[CLOSE_BUTTON][INACTIVE], theme,
+    myPixmapLoad (md, &params.buttons[CLOSE_BUTTON][INACTIVE], theme,
         "close-inactive", colsym, 20);
-    myPixmapLoad (dpy, &params.buttons[CLOSE_BUTTON][PRESSED], theme,
+    myPixmapLoad (md, &params.buttons[CLOSE_BUTTON][PRESSED], theme,
         "close-pressed", colsym, 20);
-    myPixmapLoad (dpy, &params.buttons[MAXIMIZE_BUTTON][ACTIVE], theme,
+    myPixmapLoad (md, &params.buttons[MAXIMIZE_BUTTON][ACTIVE], theme,
         "maximize-active", colsym, 20);
-    myPixmapLoad (dpy, &params.buttons[MAXIMIZE_BUTTON][INACTIVE], theme,
+    myPixmapLoad (md, &params.buttons[MAXIMIZE_BUTTON][INACTIVE], theme,
         "maximize-inactive", colsym, 20);
-    myPixmapLoad (dpy, &params.buttons[MAXIMIZE_BUTTON][PRESSED], theme,
+    myPixmapLoad (md, &params.buttons[MAXIMIZE_BUTTON][PRESSED], theme,
         "maximize-pressed", colsym, 20);
-    myPixmapLoad (dpy, &params.buttons[SHADE_BUTTON][ACTIVE], theme,
+    myPixmapLoad (md, &params.buttons[SHADE_BUTTON][ACTIVE], theme,
         "shade-active", colsym, 20);
-    myPixmapLoad (dpy, &params.buttons[SHADE_BUTTON][INACTIVE], theme,
+    myPixmapLoad (md, &params.buttons[SHADE_BUTTON][INACTIVE], theme,
         "shade-inactive", colsym, 20);
-    myPixmapLoad (dpy, &params.buttons[SHADE_BUTTON][PRESSED], theme,
+    myPixmapLoad (md, &params.buttons[SHADE_BUTTON][PRESSED], theme,
         "shade-pressed", colsym, 20);
-    myPixmapLoad (dpy, &params.buttons[STICK_BUTTON][ACTIVE], theme,
+    myPixmapLoad (md, &params.buttons[STICK_BUTTON][ACTIVE], theme,
         "stick-active", colsym, 20);
-    myPixmapLoad (dpy, &params.buttons[STICK_BUTTON][INACTIVE], theme,
+    myPixmapLoad (md, &params.buttons[STICK_BUTTON][INACTIVE], theme,
         "stick-inactive", colsym, 20);
-    myPixmapLoad (dpy, &params.buttons[STICK_BUTTON][PRESSED], theme,
+    myPixmapLoad (md, &params.buttons[STICK_BUTTON][PRESSED], theme,
         "stick-pressed", colsym, 20);
-    myPixmapLoad (dpy, &params.buttons[MENU_BUTTON][ACTIVE], theme,
+    myPixmapLoad (md, &params.buttons[MENU_BUTTON][ACTIVE], theme,
         "menu-active", colsym, 20);
-    myPixmapLoad (dpy, &params.buttons[MENU_BUTTON][INACTIVE], theme,
+    myPixmapLoad (md, &params.buttons[MENU_BUTTON][INACTIVE], theme,
         "menu-inactive", colsym, 20);
-    myPixmapLoad (dpy, &params.buttons[MENU_BUTTON][PRESSED], theme,
+    myPixmapLoad (md, &params.buttons[MENU_BUTTON][PRESSED], theme,
         "menu-pressed", colsym, 20);
-    myPixmapLoad (dpy, &params.buttons[SHADE_BUTTON][T_ACTIVE], theme,
+    myPixmapLoad (md, &params.buttons[SHADE_BUTTON][T_ACTIVE], theme,
         "shade-toggled-active", colsym, 20);
-    myPixmapLoad (dpy, &params.buttons[SHADE_BUTTON][T_INACTIVE], theme,
+    myPixmapLoad (md, &params.buttons[SHADE_BUTTON][T_INACTIVE], theme,
         "shade-toggled-inactive", colsym, 20);
-    myPixmapLoad (dpy, &params.buttons[SHADE_BUTTON][T_PRESSED], theme,
+    myPixmapLoad (md, &params.buttons[SHADE_BUTTON][T_PRESSED], theme,
         "shade-toggled-pressed", colsym, 20);
-    myPixmapLoad (dpy, &params.buttons[STICK_BUTTON][T_ACTIVE], theme,
+    myPixmapLoad (md, &params.buttons[STICK_BUTTON][T_ACTIVE], theme,
         "stick-toggled-active", colsym, 20);
-    myPixmapLoad (dpy, &params.buttons[STICK_BUTTON][T_INACTIVE], theme,
+    myPixmapLoad (md, &params.buttons[STICK_BUTTON][T_INACTIVE], theme,
         "stick-toggled-inactive", colsym, 20);
-    myPixmapLoad (dpy, &params.buttons[STICK_BUTTON][T_PRESSED], theme,
+    myPixmapLoad (md, &params.buttons[STICK_BUTTON][T_PRESSED], theme,
         "stick-toggled-pressed", colsym, 20);
-    myPixmapLoad (dpy, &params.buttons[MAXIMIZE_BUTTON][T_ACTIVE], theme,
+    myPixmapLoad (md, &params.buttons[MAXIMIZE_BUTTON][T_ACTIVE], theme,
         "maximize-toggled-active", colsym, 20);
-    myPixmapLoad (dpy, &params.buttons[MAXIMIZE_BUTTON][T_INACTIVE], theme,
+    myPixmapLoad (md, &params.buttons[MAXIMIZE_BUTTON][T_INACTIVE], theme,
         "maximize-toggled-inactive", colsym, 20);
-    myPixmapLoad (dpy, &params.buttons[MAXIMIZE_BUTTON][T_PRESSED], theme,
+    myPixmapLoad (md, &params.buttons[MAXIMIZE_BUTTON][T_PRESSED], theme,
         "maximize-toggled-pressed", colsym, 20);
-    myPixmapLoad (dpy, &params.title[TITLE_1][ACTIVE], theme,
+    myPixmapLoad (md, &params.title[TITLE_1][ACTIVE], theme,
         "title-1-active", colsym, 20);
-    myPixmapLoad (dpy, &params.title[TITLE_1][INACTIVE], theme,
+    myPixmapLoad (md, &params.title[TITLE_1][INACTIVE], theme,
         "title-1-inactive", colsym, 20);
-    myPixmapLoad (dpy, &params.title[TITLE_2][ACTIVE], theme,
+    myPixmapLoad (md, &params.title[TITLE_2][ACTIVE], theme,
         "title-2-active", colsym, 20);
-    myPixmapLoad (dpy, &params.title[TITLE_2][INACTIVE], theme,
+    myPixmapLoad (md, &params.title[TITLE_2][INACTIVE], theme,
         "title-2-inactive", colsym, 20);
-    myPixmapLoad (dpy, &params.title[TITLE_3][ACTIVE], theme,
+    myPixmapLoad (md, &params.title[TITLE_3][ACTIVE], theme,
         "title-3-active", colsym, 20);
-    myPixmapLoad (dpy, &params.title[TITLE_3][INACTIVE], theme,
+    myPixmapLoad (md, &params.title[TITLE_3][INACTIVE], theme,
         "title-3-inactive", colsym, 20);
-    myPixmapLoad (dpy, &params.title[TITLE_4][ACTIVE], theme,
+    myPixmapLoad (md, &params.title[TITLE_4][ACTIVE], theme,
         "title-4-active", colsym, 20);
-    myPixmapLoad (dpy, &params.title[TITLE_4][INACTIVE], theme,
+    myPixmapLoad (md, &params.title[TITLE_4][INACTIVE], theme,
         "title-4-inactive", colsym, 20);
-    myPixmapLoad (dpy, &params.title[TITLE_5][ACTIVE], theme,
+    myPixmapLoad (md, &params.title[TITLE_5][ACTIVE], theme,
         "title-5-active", colsym, 20);
-    myPixmapLoad (dpy, &params.title[TITLE_5][INACTIVE], theme,
+    myPixmapLoad (md, &params.title[TITLE_5][INACTIVE], theme,
         "title-5-inactive", colsym, 20);
 
     if (!g_ascii_strcasecmp ("left", getValue ("title_alignment", rc)))
@@ -753,7 +751,7 @@ loadTheme (Settings rc[])
     params.title_horizontal_offset =
         TOINT (getValue ("title_horizontal_offset", rc));
 
-    params.box_gc = createGC (cmap, "#FFFFFF", GXxor, NULL, 2, TRUE);
+    params.box_gc = createGC (md, "#FFFFFF", GXxor, NULL, 2, TRUE);
 
     g_free (theme);
 }
@@ -782,7 +780,7 @@ loadShortcutCmd (Settings rc[])
 }
 
 static gboolean
-loadKeyBindings (Settings rc[])
+loadKeyBindings (ScreenData *md, Settings rc[])
 {
     gchar *keytheme;
     gchar *keythemevalue;
@@ -794,7 +792,7 @@ loadKeyBindings (Settings rc[])
         if (!parseRc (KEYTHEMERC, keytheme, rc))
         {
             g_warning (_("%s: specified key theme \"%s\" missing, using default"),
-                       progname, keythemevalue);
+                       g_get_prgname (), keythemevalue);
             g_free (keytheme);
             keytheme = getThemeDir (DEFAULT_KEYTHEME, KEYTHEMERC);
             parseRc (KEYTHEMERC, keytheme, rc);
@@ -803,148 +801,148 @@ loadKeyBindings (Settings rc[])
 
         if (!checkRc (rc))
         {
-            g_warning (_("%s: Missing values in defaults file"), progname);
+            g_warning (_("%s: Missing values in defaults file"), g_get_prgname ());
             return FALSE;
         }
     }
 
     loadShortcutCmd (rc);
 
-    parseKeyString (dpy, &params.keys[KEY_MOVE_UP],
+    parseKeyString (md->dpy, &params.keys[KEY_MOVE_UP],
         getValue ("move_window_up_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_MOVE_DOWN],
+    parseKeyString (md->dpy, &params.keys[KEY_MOVE_DOWN],
         getValue ("move_window_down_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_MOVE_LEFT],
+    parseKeyString (md->dpy, &params.keys[KEY_MOVE_LEFT],
         getValue ("move_window_left_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_MOVE_RIGHT],
+    parseKeyString (md->dpy, &params.keys[KEY_MOVE_RIGHT],
         getValue ("move_window_right_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_RESIZE_UP],
+    parseKeyString (md->dpy, &params.keys[KEY_RESIZE_UP],
         getValue ("resize_window_up_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_RESIZE_DOWN],
+    parseKeyString (md->dpy, &params.keys[KEY_RESIZE_DOWN],
         getValue ("resize_window_down_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_RESIZE_LEFT],
+    parseKeyString (md->dpy, &params.keys[KEY_RESIZE_LEFT],
         getValue ("resize_window_left_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_RESIZE_RIGHT],
+    parseKeyString (md->dpy, &params.keys[KEY_RESIZE_RIGHT],
         getValue ("resize_window_right_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_CYCLE_WINDOWS],
+    parseKeyString (md->dpy, &params.keys[KEY_CYCLE_WINDOWS],
         getValue ("cycle_windows_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_CLOSE_WINDOW],
+    parseKeyString (md->dpy, &params.keys[KEY_CLOSE_WINDOW],
         getValue ("close_window_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_HIDE_WINDOW],
+    parseKeyString (md->dpy, &params.keys[KEY_HIDE_WINDOW],
         getValue ("hide_window_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_MAXIMIZE_WINDOW],
+    parseKeyString (md->dpy, &params.keys[KEY_MAXIMIZE_WINDOW],
         getValue ("maximize_window_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_MAXIMIZE_VERT],
+    parseKeyString (md->dpy, &params.keys[KEY_MAXIMIZE_VERT],
         getValue ("maximize_vert_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_MAXIMIZE_HORIZ],
+    parseKeyString (md->dpy, &params.keys[KEY_MAXIMIZE_HORIZ],
         getValue ("maximize_horiz_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_SHADE_WINDOW],
+    parseKeyString (md->dpy, &params.keys[KEY_SHADE_WINDOW],
         getValue ("shade_window_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_NEXT_WORKSPACE],
+    parseKeyString (md->dpy, &params.keys[KEY_NEXT_WORKSPACE],
         getValue ("next_workspace_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_PREV_WORKSPACE],
+    parseKeyString (md->dpy, &params.keys[KEY_PREV_WORKSPACE],
         getValue ("prev_workspace_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_ADD_WORKSPACE],
+    parseKeyString (md->dpy, &params.keys[KEY_ADD_WORKSPACE],
         getValue ("add_workspace_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_DEL_WORKSPACE],
+    parseKeyString (md->dpy, &params.keys[KEY_DEL_WORKSPACE],
         getValue ("del_workspace_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_STICK_WINDOW],
+    parseKeyString (md->dpy, &params.keys[KEY_STICK_WINDOW],
         getValue ("stick_window_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_WORKSPACE_1],
+    parseKeyString (md->dpy, &params.keys[KEY_WORKSPACE_1],
         getValue ("workspace_1_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_WORKSPACE_2],
+    parseKeyString (md->dpy, &params.keys[KEY_WORKSPACE_2],
         getValue ("workspace_2_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_WORKSPACE_3],
+    parseKeyString (md->dpy, &params.keys[KEY_WORKSPACE_3],
         getValue ("workspace_3_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_WORKSPACE_4],
+    parseKeyString (md->dpy, &params.keys[KEY_WORKSPACE_4],
         getValue ("workspace_4_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_WORKSPACE_5],
+    parseKeyString (md->dpy, &params.keys[KEY_WORKSPACE_5],
         getValue ("workspace_5_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_WORKSPACE_6],
+    parseKeyString (md->dpy, &params.keys[KEY_WORKSPACE_6],
         getValue ("workspace_6_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_WORKSPACE_7],
+    parseKeyString (md->dpy, &params.keys[KEY_WORKSPACE_7],
         getValue ("workspace_7_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_WORKSPACE_8],
+    parseKeyString (md->dpy, &params.keys[KEY_WORKSPACE_8],
         getValue ("workspace_8_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_WORKSPACE_9],
+    parseKeyString (md->dpy, &params.keys[KEY_WORKSPACE_9],
         getValue ("workspace_9_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_MOVE_NEXT_WORKSPACE],
+    parseKeyString (md->dpy, &params.keys[KEY_MOVE_NEXT_WORKSPACE],
         getValue ("move_window_next_workspace_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_MOVE_PREV_WORKSPACE],
+    parseKeyString (md->dpy, &params.keys[KEY_MOVE_PREV_WORKSPACE],
         getValue ("move_window_prev_workspace_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_MOVE_WORKSPACE_1],
+    parseKeyString (md->dpy, &params.keys[KEY_MOVE_WORKSPACE_1],
         getValue ("move_window_workspace_1_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_MOVE_WORKSPACE_2],
+    parseKeyString (md->dpy, &params.keys[KEY_MOVE_WORKSPACE_2],
         getValue ("move_window_workspace_2_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_MOVE_WORKSPACE_3],
+    parseKeyString (md->dpy, &params.keys[KEY_MOVE_WORKSPACE_3],
         getValue ("move_window_workspace_3_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_MOVE_WORKSPACE_4],
+    parseKeyString (md->dpy, &params.keys[KEY_MOVE_WORKSPACE_4],
         getValue ("move_window_workspace_4_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_MOVE_WORKSPACE_5],
+    parseKeyString (md->dpy, &params.keys[KEY_MOVE_WORKSPACE_5],
         getValue ("move_window_workspace_5_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_MOVE_WORKSPACE_6],
+    parseKeyString (md->dpy, &params.keys[KEY_MOVE_WORKSPACE_6],
         getValue ("move_window_workspace_6_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_MOVE_WORKSPACE_7],
+    parseKeyString (md->dpy, &params.keys[KEY_MOVE_WORKSPACE_7],
         getValue ("move_window_workspace_7_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_MOVE_WORKSPACE_8],
+    parseKeyString (md->dpy, &params.keys[KEY_MOVE_WORKSPACE_8],
         getValue ("move_window_workspace_8_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_MOVE_WORKSPACE_9],
+    parseKeyString (md->dpy, &params.keys[KEY_MOVE_WORKSPACE_9],
         getValue ("move_window_workspace_9_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_SHORTCUT_1],
+    parseKeyString (md->dpy, &params.keys[KEY_SHORTCUT_1],
         getValue ("shortcut_1_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_SHORTCUT_2],
+    parseKeyString (md->dpy, &params.keys[KEY_SHORTCUT_2],
         getValue ("shortcut_2_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_SHORTCUT_3],
+    parseKeyString (md->dpy, &params.keys[KEY_SHORTCUT_3],
         getValue ("shortcut_3_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_SHORTCUT_4],
+    parseKeyString (md->dpy, &params.keys[KEY_SHORTCUT_4],
         getValue ("shortcut_4_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_SHORTCUT_5],
+    parseKeyString (md->dpy, &params.keys[KEY_SHORTCUT_5],
         getValue ("shortcut_5_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_SHORTCUT_6],
+    parseKeyString (md->dpy, &params.keys[KEY_SHORTCUT_6],
         getValue ("shortcut_6_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_SHORTCUT_7],
+    parseKeyString (md->dpy, &params.keys[KEY_SHORTCUT_7],
         getValue ("shortcut_7_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_SHORTCUT_8],
+    parseKeyString (md->dpy, &params.keys[KEY_SHORTCUT_8],
         getValue ("shortcut_8_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_SHORTCUT_9],
+    parseKeyString (md->dpy, &params.keys[KEY_SHORTCUT_9],
         getValue ("shortcut_9_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_SHORTCUT_10],
+    parseKeyString (md->dpy, &params.keys[KEY_SHORTCUT_10],
         getValue ("shortcut_10_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_RAISE_WINDOW],
+    parseKeyString (md->dpy, &params.keys[KEY_RAISE_WINDOW],
         getValue ("raise_window_key", rc));
-    parseKeyString (dpy, &params.keys[KEY_LOWER_WINDOW],
+    parseKeyString (md->dpy, &params.keys[KEY_LOWER_WINDOW],
         getValue ("lower_window_key", rc));
-    ungrabKeys (dpy, gnome_win);
-    grabKey (dpy, &params.keys[KEY_CYCLE_WINDOWS], gnome_win);
-    grabKey (dpy, &params.keys[KEY_NEXT_WORKSPACE], gnome_win);
-    grabKey (dpy, &params.keys[KEY_PREV_WORKSPACE], gnome_win);
-    grabKey (dpy, &params.keys[KEY_ADD_WORKSPACE], gnome_win);
-    grabKey (dpy, &params.keys[KEY_NEXT_WORKSPACE], gnome_win);
-    grabKey (dpy, &params.keys[KEY_WORKSPACE_1], gnome_win);
-    grabKey (dpy, &params.keys[KEY_WORKSPACE_2], gnome_win);
-    grabKey (dpy, &params.keys[KEY_WORKSPACE_3], gnome_win);
-    grabKey (dpy, &params.keys[KEY_WORKSPACE_4], gnome_win);
-    grabKey (dpy, &params.keys[KEY_WORKSPACE_5], gnome_win);
-    grabKey (dpy, &params.keys[KEY_WORKSPACE_6], gnome_win);
-    grabKey (dpy, &params.keys[KEY_WORKSPACE_7], gnome_win);
-    grabKey (dpy, &params.keys[KEY_WORKSPACE_8], gnome_win);
-    grabKey (dpy, &params.keys[KEY_WORKSPACE_9], gnome_win);
-    grabKey (dpy, &params.keys[KEY_SHORTCUT_1], gnome_win);
-    grabKey (dpy, &params.keys[KEY_SHORTCUT_2], gnome_win);
-    grabKey (dpy, &params.keys[KEY_SHORTCUT_3], gnome_win);
-    grabKey (dpy, &params.keys[KEY_SHORTCUT_4], gnome_win);
-    grabKey (dpy, &params.keys[KEY_SHORTCUT_5], gnome_win);
-    grabKey (dpy, &params.keys[KEY_SHORTCUT_6], gnome_win);
-    grabKey (dpy, &params.keys[KEY_SHORTCUT_7], gnome_win);
-    grabKey (dpy, &params.keys[KEY_SHORTCUT_8], gnome_win);
-    grabKey (dpy, &params.keys[KEY_SHORTCUT_9], gnome_win);
-    grabKey (dpy, &params.keys[KEY_SHORTCUT_10], gnome_win);
+    ungrabKeys (md->dpy, md->gnome_win);
+    grabKey (md->dpy, &params.keys[KEY_CYCLE_WINDOWS], md->gnome_win);
+    grabKey (md->dpy, &params.keys[KEY_NEXT_WORKSPACE], md->gnome_win);
+    grabKey (md->dpy, &params.keys[KEY_PREV_WORKSPACE], md->gnome_win);
+    grabKey (md->dpy, &params.keys[KEY_ADD_WORKSPACE], md->gnome_win);
+    grabKey (md->dpy, &params.keys[KEY_NEXT_WORKSPACE], md->gnome_win);
+    grabKey (md->dpy, &params.keys[KEY_WORKSPACE_1], md->gnome_win);
+    grabKey (md->dpy, &params.keys[KEY_WORKSPACE_2], md->gnome_win);
+    grabKey (md->dpy, &params.keys[KEY_WORKSPACE_3], md->gnome_win);
+    grabKey (md->dpy, &params.keys[KEY_WORKSPACE_4], md->gnome_win);
+    grabKey (md->dpy, &params.keys[KEY_WORKSPACE_5], md->gnome_win);
+    grabKey (md->dpy, &params.keys[KEY_WORKSPACE_6], md->gnome_win);
+    grabKey (md->dpy, &params.keys[KEY_WORKSPACE_7], md->gnome_win);
+    grabKey (md->dpy, &params.keys[KEY_WORKSPACE_8], md->gnome_win);
+    grabKey (md->dpy, &params.keys[KEY_WORKSPACE_9], md->gnome_win);
+    grabKey (md->dpy, &params.keys[KEY_SHORTCUT_1], md->gnome_win);
+    grabKey (md->dpy, &params.keys[KEY_SHORTCUT_2], md->gnome_win);
+    grabKey (md->dpy, &params.keys[KEY_SHORTCUT_3], md->gnome_win);
+    grabKey (md->dpy, &params.keys[KEY_SHORTCUT_4], md->gnome_win);
+    grabKey (md->dpy, &params.keys[KEY_SHORTCUT_5], md->gnome_win);
+    grabKey (md->dpy, &params.keys[KEY_SHORTCUT_6], md->gnome_win);
+    grabKey (md->dpy, &params.keys[KEY_SHORTCUT_7], md->gnome_win);
+    grabKey (md->dpy, &params.keys[KEY_SHORTCUT_8], md->gnome_win);
+    grabKey (md->dpy, &params.keys[KEY_SHORTCUT_9], md->gnome_win);
+    grabKey (md->dpy, &params.keys[KEY_SHORTCUT_10], md->gnome_win);
 
     return TRUE;
 }
 
 gboolean
-loadSettings (void)
+loadSettings (ScreenData *md)
 {
     Settings rc[] = {
         /* Do not chnage the order of the following parameters */
@@ -1074,11 +1072,11 @@ loadSettings (void)
 
     TRACE ("entering loadSettings");
 
-    loadRcData (rc);
-    loadMcsData (rc);
-    loadTheme (rc);
+    loadRcData (md, rc);
+    loadMcsData (md, rc);
+    loadTheme (md, rc);
 
-    if (!loadKeyBindings (rc))
+    if (!loadKeyBindings (md, rc))
     {
         freeRc (rc);
         return FALSE;
@@ -1111,11 +1109,10 @@ loadSettings (void)
     params.snap_width = abs (TOINT (getValue ("snap_width", rc)));
     params.dbl_click_time = abs (TOINT (getValue ("dbl_click_time", rc)));
 
-    set_settings_margin (LEFT, TOINT (getValue ("margin_left", rc)));
-    set_settings_margin (RIGHT, TOINT (getValue ("margin_right", rc)));
-    set_settings_margin (BOTTOM, TOINT (getValue ("margin_bottom",
-                rc)));
-    set_settings_margin (TOP, TOINT (getValue ("margin_top", rc)));
+    set_settings_margin (md, LEFT,   TOINT (getValue ("margin_left", rc)));
+    set_settings_margin (md, RIGHT,  TOINT (getValue ("margin_right", rc)));
+    set_settings_margin (md, BOTTOM, TOINT (getValue ("margin_bottom", rc)));
+    set_settings_margin (md, TOP,    TOINT (getValue ("margin_top", rc)));
 
     g_value_init (&tmp_val, G_TYPE_INT);
     if (gdk_setting_get ("gtk-double-click-time", &tmp_val))
@@ -1152,7 +1149,7 @@ loadSettings (void)
         }
         g_message (_("%s: Workspace count not set, using rc value: %i"),
             g_get_prgname (), workspace_count);
-        workspaceSetCount (workspace_count);
+        workspaceSetCount (md, workspace_count);
     }
 
     params.wrap_workspaces =
@@ -1166,62 +1163,62 @@ loadSettings (void)
 }
 
 static void
-unloadTheme (void)
+unloadTheme (ScreenData *md)
 {
     int i;
     TRACE ("entering unloadTheme");
 
     for (i = 0; i < 3; i++)
     {
-        myPixmapFree (dpy, &params.sides[i][ACTIVE]);
-        myPixmapFree (dpy, &params.sides[i][INACTIVE]);
+        myPixmapFree (&params.sides[i][ACTIVE]);
+        myPixmapFree (&params.sides[i][INACTIVE]);
     }
     for (i = 0; i < 4; i++)
     {
-        myPixmapFree (dpy, &params.corners[i][ACTIVE]);
-        myPixmapFree (dpy, &params.corners[i][INACTIVE]);
+        myPixmapFree (&params.corners[i][ACTIVE]);
+        myPixmapFree (&params.corners[i][INACTIVE]);
     }
     for (i = 0; i < BUTTON_COUNT; i++)
     {
-        myPixmapFree (dpy, &params.buttons[i][ACTIVE]);
-        myPixmapFree (dpy, &params.buttons[i][INACTIVE]);
-        myPixmapFree (dpy, &params.buttons[i][PRESSED]);
-        myPixmapFree (dpy, &params.buttons[i][T_ACTIVE]);
-        myPixmapFree (dpy, &params.buttons[i][T_INACTIVE]);
-        myPixmapFree (dpy, &params.buttons[i][T_PRESSED]);
+        myPixmapFree (&params.buttons[i][ACTIVE]);
+        myPixmapFree (&params.buttons[i][INACTIVE]);
+        myPixmapFree (&params.buttons[i][PRESSED]);
+        myPixmapFree (&params.buttons[i][T_ACTIVE]);
+        myPixmapFree (&params.buttons[i][T_INACTIVE]);
+        myPixmapFree (&params.buttons[i][T_PRESSED]);
     }
     for (i = 0; i < 5; i++)
     {
-        myPixmapFree (dpy, &params.title[i][ACTIVE]);
-        myPixmapFree (dpy, &params.title[i][INACTIVE]);
+        myPixmapFree (&params.title[i][ACTIVE]);
+        myPixmapFree (&params.title[i][INACTIVE]);
     }
     if (params.box_gc != None)
     {
-        XFreeGC (dpy, params.box_gc);
+        XFreeGC (md->dpy, params.box_gc);
         params.box_gc = None;
     }
 }
 
 gboolean
-reloadSettings (int mask)
+reloadSettings (ScreenData *md, int mask)
 {
     TRACE ("entering reloadSettings");
 
-    unloadTheme ();
-    if (!loadSettings ())
+    unloadTheme (md);
+    if (!loadSettings (md))
     {
         return FALSE;
     }
     if (mask)
     {
-        clientUpdateAllFrames (mask);
+        clientUpdateAllFrames (md, mask);
     }
 
     return TRUE;
 }
 
 gboolean
-initSettings (void)
+initSettings (ScreenData *md)
 {
     int i;
     long val = 0;
@@ -1267,11 +1264,11 @@ initSettings (void)
         myPixmapInit (&params.title[i][ACTIVE]);
         myPixmapInit (&params.title[i][INACTIVE]);
     }
-    if (!mcs_client_check_manager (dpy, screen, "xfce-mcs-manager"))
+    if (!mcs_client_check_manager (md->dpy, md->screen, "xfce-mcs-manager"))
     {
         g_warning ("MCS manager not running");
     }
-    client = mcs_client_new (dpy, screen, notify_cb, watch_cb, NULL);
+    client = mcs_client_new (md->dpy, md->screen, notify_cb, watch_cb, (gpointer) md);
     if (client)
     {
         mcs_client_add_channel (client, CHANNEL1);
@@ -1285,16 +1282,16 @@ initSettings (void)
 
     mcs_initted = TRUE;
 
-    if (getHint (dpy, root, net_number_of_desktops, &val))
+    if (getHint (md->dpy, md->xroot, net_number_of_desktops, &val))
     {
-        workspaceSetCount (val);
+        workspaceSetCount (md, val);
     }
-    else if (getHint (dpy, root, win_workspace_count, &val))
+    else if (getHint (md->dpy, md->xroot, win_workspace_count, &val))
     {
-        workspaceSetCount (val);
+        workspaceSetCount (md, val);
     }
 
-    if (getUTF8String (dpy, root, net_desktop_names, &names, &i))
+    if (getUTF8String (md->dpy, md->xroot, net_desktop_names, &names, &i))
     {
         workspaceSetNames (names, i);
     }
@@ -1304,19 +1301,19 @@ initSettings (void)
         params.workspace_names_length = 0;
     }
 
-    if (!loadSettings ())
+    if (!loadSettings (md))
     {
         return FALSE;
     }
-    placeSidewalks (params.wrap_workspaces);
+    placeSidewalks (md, params.wrap_workspaces);
 
     return TRUE;
 }
 
 void
-unloadSettings (void)
+unloadSettings (ScreenData *md)
 {
     TRACE ("entering unloadSettings");
 
-    unloadTheme ();
+    unloadTheme (md);
 }
