@@ -1,8 +1,8 @@
 /*
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
-        the Free Software Foundation; You may only use version 2 of the License,
-        you have no option to use any other version.
+        the Free Software Foundation; either version 2, or (at your option)
+        any later version.
  
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,7 +14,7 @@
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2003 Olivier Fourdan
+        xfwm4    - (c) 2002-2004 Olivier Fourdan
  
  */
 
@@ -25,10 +25,12 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xmd.h>
-#include <gtk/gtk.h>
 #include <glib.h>
-#include <libxfce4util/debug.h>
-#include "main.h"
+#include <gdk/gdk.h>
+#include <gtk/gtk.h>
+#include <libxfce4util/libxfce4util.h> 
+
+#include "screen.h"
 #include "misc.h"
 #include "transients.h"
 #include "workspaces.h"
@@ -39,7 +41,7 @@
 #include "hints.h"
 
 void
-workspaceSwitch (int new_ws, Client * c2)
+workspaceSwitch (ScreenData *md, int new_ws, Client * c2)
 {
     Client *c, *new_focus = NULL;
     Client *previous;
@@ -61,16 +63,16 @@ workspaceSwitch (int new_ws, Client * c2)
         new_ws = params.workspace_count - 1;
     }
 
-    if (new_ws == workspace)
+    if (new_ws == md->current_ws)
     {
         return;
     }
 
     /* Grab the pointer to avoid side effects with EnterNotify events */
-    XGrabPointer (dpy, gnome_win, FALSE, EnterWindowMask, GrabModeAsync,
-                       GrabModeAsync, None, None, CurrentTime);
+    XGrabPointer (md->dpy, md->gnome_win, FALSE, EnterWindowMask, GrabModeAsync,
+                       GrabModeAsync, None, None, GDK_CURRENT_TIME);
 
-    workspace = new_ws;
+    md->current_ws = new_ws;
     if (c2)
     {
         clientSetWorkspace (c2, new_ws, FALSE);
@@ -89,7 +91,7 @@ workspaceSwitch (int new_ws, Client * c2)
             if (c == previous)
             {
                 FLAG_SET (previous->flags, CLIENT_FLAG_FOCUS);
-                clientSetFocus (NULL, FOCUS_IGNORE_MODAL);
+                clientSetFocus (md, NULL, GDK_CURRENT_TIME, FOCUS_IGNORE_MODAL);
             }
             if (!clientIsTransientOrModal (c))
             {
@@ -142,18 +144,18 @@ workspaceSwitch (int new_ws, Client * c2)
         }
     }
 
-    setHint (dpy, root, win_workspace, new_ws);
+    setHint (md->dpy, md->xroot, win_workspace, new_ws);
     data[0] = new_ws;
-    XChangeProperty (dpy, root, net_current_desktop, XA_CARDINAL, 32,
+    XChangeProperty (md->dpy, md->xroot, net_current_desktop, XA_CARDINAL, 32,
         PropModeReplace, (unsigned char *) data, 1);
-    workspaceUpdateArea (margins, gnome_margins);
+    workspaceUpdateArea (md);
     
     /* Ungrab the pointer we grabbed before mapping/unmapping all windows */
-    XUngrabPointer (dpy, CurrentTime);
+    XUngrabPointer (md->dpy, GDK_CURRENT_TIME);
 
     if (!(params.click_to_focus))
     {
-        if (!(c2) && (XQueryPointer (dpy, root, &dr, &window, &rx, &ry, &wx, &wy, &mask)))
+        if (!(c2) && (XQueryPointer (md->dpy, md->xroot, &dr, &window, &rx, &ry, &wx, &wy, &mask)))
         {
             c = clientAtPosition (rx, ry, NULL);
             if (c)
@@ -162,7 +164,10 @@ workspaceSwitch (int new_ws, Client * c2)
             }
         }
     }
-    clientSetFocus (new_focus, NO_FOCUS_FLAG);
+    if (new_focus)
+    {
+        clientSetFocus (new_focus->md, new_focus, GDK_CURRENT_TIME, NO_FOCUS_FLAG);
+    }
 }
 
 void
@@ -178,7 +183,7 @@ workspaceSetNames (char *names, int length)
 }
 
 void
-workspaceSetCount (int count)
+workspaceSetCount (ScreenData * md, int count)
 {
     Client *c;
     int i;
@@ -194,8 +199,8 @@ workspaceSetCount (int count)
         return;
     }
 
-    setHint (dpy, root, win_workspace_count, count);
-    setHint (dpy, root, net_number_of_desktops, count);
+    setHint (md->dpy, md->xroot, win_workspace_count, count);
+    setHint (md->dpy, md->xroot, net_number_of_desktops, count);
     params.workspace_count = count;
 
     for (c = clients, i = 0; i < client_count; c = c->next, i++)
@@ -205,78 +210,61 @@ workspaceSetCount (int count)
             clientSetWorkspace (c, count - 1, TRUE);
         }
     }
-    if (workspace > count - 1)
+    if (md->current_ws > count - 1)
     {
-        workspaceSwitch (count - 1, NULL);
+        workspaceSwitch (md, count - 1, NULL);
     }
-    setNetWorkarea (dpy, screen, params.workspace_count, margins);
+    setNetWorkarea (md->dpy, md->screen, params.workspace_count, 
+                         gdk_screen_get_width (md->gscr),
+                         gdk_screen_get_height (md->gscr),
+                         md->margins);
 }
 
 void
-workspaceGetArea (int * m1, int * m2, Client * c)
-{
-    Client *c2;
-    int i;
-
-    TRACE ("entering workspaceGetArea");
-
-    g_return_if_fail (m1 != NULL);
-
-    for (i = 0; i < 4; i++)
-    {
-        if (m2 == NULL)
-        {
-            m1[i] = 0;
-        }
-        else
-        {
-            m1[i] = m2[i];
-        }
-    }
-
-    for (c2 = clients, i = 0; i < client_count; c2 = c2->next, i++)
-    {
-        if (((!c) || (c != c2))
-            && FLAG_TEST_ALL (c2->flags,
-                CLIENT_FLAG_HAS_STRUT | CLIENT_FLAG_VISIBLE))
-        {
-            m1[TOP] = MAX (m1[TOP], c2->struts[TOP]);
-            m1[LEFT] = MAX (m1[LEFT], c2->struts[LEFT]);
-            m1[RIGHT] =
-                MAX (m1[RIGHT], c2->struts[RIGHT]);
-            m1[BOTTOM] =
-                MAX (m1[BOTTOM], c2->struts[BOTTOM]);
-        }
-    }
-}
-
-void
-workspaceUpdateArea (int * m1, int * m2)
+workspaceUpdateArea (ScreenData *md)
 {
+    Client *c;
     int prev_top;
     int prev_left;
     int prev_right;
     int prev_bottom;
+    int i;
 
-    g_return_if_fail (m1 != NULL);
-    g_return_if_fail (m2 != NULL);
+    g_return_if_fail (md != NULL);
+    g_return_if_fail (md->margins != NULL);
+    g_return_if_fail (md->gnome_margins != NULL);
 
     TRACE ("entering workspaceUpdateArea");
 
-    prev_top = m1[TOP];
-    prev_left = m1[LEFT];
-    prev_right = m1[RIGHT];
-    prev_bottom = m1[BOTTOM];
+    prev_top = md->margins[TOP];
+    prev_left = md->margins[LEFT];
+    prev_right = md->margins[RIGHT];
+    prev_bottom = md->margins[BOTTOM];
 
-    workspaceGetArea (m1, m2, NULL);
+    for (i = 0; i < 4; i++)
+    {
+        md->margins[i] = md->gnome_margins[i];
+    }
 
-    TRACE ("Desktop area computed : (%d,%d,%d,%d)", (int) m1[0], (int) m1[1],
-        (int) m1[2], (int) m1[3]);
-    if ((prev_top != m1[TOP]) || (prev_left != m1[LEFT])
-        || (prev_right != m1[RIGHT])
-        || (prev_bottom != m1[BOTTOM]))
+    for (c = clients, i = 0; i < client_count; c = c->next, i++)
+    {
+        if (FLAG_TEST_ALL (c->flags, CLIENT_FLAG_HAS_STRUT | CLIENT_FLAG_VISIBLE))
+        {
+            md->margins[TOP]    = MAX (md->margins[TOP],    c->struts[TOP]);
+            md->margins[LEFT]   = MAX (md->margins[LEFT],   c->struts[LEFT]);
+            md->margins[RIGHT]  = MAX (md->margins[RIGHT],  c->struts[RIGHT]);
+            md->margins[BOTTOM] = MAX (md->margins[BOTTOM], c->struts[BOTTOM]);
+        }
+    }
+
+    if ((prev_top != md->margins[TOP]) || (prev_left != md->margins[LEFT])
+        || (prev_right != md->margins[RIGHT])
+        || (prev_bottom != md->margins[BOTTOM]))
     {
         TRACE ("Margins have changed, updating net_workarea");
-        setNetWorkarea (dpy, screen, params.workspace_count, m1);
+        setNetWorkarea (md->dpy, md->screen, params.workspace_count, 
+                             gdk_screen_get_width (md->gscr),
+                             gdk_screen_get_height (md->gscr),
+                             md->margins);
     }
 }
