@@ -1,8 +1,8 @@
 /*
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
-        the Free Software Foundation; You may only use version 2 of the License,
-        you have no option to use any other version.
+        the Free Software Foundation; either version 2, or (at your option)
+        any later version.
  
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,7 +14,7 @@
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2003 Olivier Fourdan
+        xfwm4    - (c) 2002-2004 Olivier Fourdan
  
  */
 
@@ -28,12 +28,12 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xmd.h>
+#include "screen.h"
 #include "client.h"
 
-void workspaceSwitch (int, Client *);
+void workspaceSwitch (ScreenData *, int, Client *);
 void workspaceSetNames (char *, int);
-void workspaceSetCount (int);
-void workspaceGetArea (int *, int *, Client *);
-void workspaceUpdateArea (int *, int *);
+void workspaceSetCount (ScreenData *, int);
+void workspaceUpdateArea (ScreenData *);
 
 #endif /* INC_WORKSPACES_H */
